@@ -1,21 +1,21 @@
 /*
- * This file is part of oFono - Open Source Telephony
  *
- * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+ *  oFono - Open Source Telephony
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * version 2 as published by the Free Software Foundation.
+ *  Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -50,19 +50,20 @@ struct cbs_data {
 static void isi_set_topics(struct ofono_cbs *cbs, const char *topics,
 				ofono_cbs_set_cb_t cb, void *data)
 {
-	DBG("Not implemented (topics=%s)", topics);
-	CALLBACK_WITH_FAILURE(cb, data);
+	DBG("Not implemented (topics=%s), all topics accepted", topics);
+	CALLBACK_WITH_SUCCESS(cb, data);
 }
 
 static void isi_clear_topics(struct ofono_cbs *cbs,
 				ofono_cbs_set_cb_t cb, void *data)
 {
 	DBG("Not implemented");
-	CALLBACK_WITH_FAILURE(cb, data);
+	CALLBACK_WITH_SUCCESS(cb, data);
 }
 
-static void routing_ntf_cb(GIsiClient *client, const void *restrict data,
-				size_t len, uint16_t object, void *opaque)
+static void routing_ntf_cb(GIsiClient *client,
+				const void *restrict data, size_t len,
+				uint16_t object, void *opaque)
 {
 	const unsigned char *msg = data;
 	struct ofono_cbs *cbs = opaque;
@@ -70,22 +71,31 @@ static void routing_ntf_cb(GIsiClient *client, const void *restrict data,
 	if (!msg || len < 3 || msg[0] != SMS_GSM_CB_ROUTING_NTF)
 		return;
 
-	ofono_cbs_notify(cbs, msg+5, len-5);
+	/* Skipping header(s) */
+	msg += 5;
+	len -= 5;
+
+	/*
+	 * The next 88 bytes of the sub-block are the actual CBS PDU,
+	 * followed by an informational data length field, and filler.
+	 */
+	ofono_cbs_notify(cbs, msg, len - 2);
 }
 
-static bool routing_resp_cb(GIsiClient *client, const void *restrict data,
-				size_t len, uint16_t object, void *opaque)
+static gboolean routing_resp_cb(GIsiClient *client,
+				const void *restrict data, size_t len,
+				uint16_t object, void *opaque)
 {
 	const unsigned char *msg = data;
 	struct ofono_cbs *cbs = opaque;
 
 	if (!msg) {
 		DBG("ISI client error: %d", g_isi_client_error(client));
-		return true;
+		return TRUE;
 	}
 
 	if (len < 3 || msg[0] != SMS_GSM_CB_ROUTING_RESP)
-		return false;
+		return FALSE;
 
 	if (msg[1] != SMS_OK) {
 		if (msg[1] == SMS_ERR_PP_RESERVED)
@@ -96,14 +106,14 @@ static bool routing_resp_cb(GIsiClient *client, const void *restrict data,
 				"routing endpoint.\n  As a consequence, "
 				"receiving CBSs is NOT going to work.\n\n",
 				msg[1], sms_isi_cause_name(msg[1]));
-		return true;
+		return TRUE;
 	}
 
 	g_isi_subscribe(client, SMS_GSM_CB_ROUTING_NTF, routing_ntf_cb,
 			cbs);
 
 	ofono_cbs_register(cbs);
-	return true;
+	return TRUE;
 }
 
 static int isi_cbs_probe(struct ofono_cbs *cbs, unsigned int vendor,
