@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -27,6 +27,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
 
 #include <glib.h>
 
@@ -92,6 +93,8 @@ static void mbm_e2ipcfg_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	const char *interface;
 	gboolean success = FALSE;
 
+	DBG("ok %d", ok);
+
 	if (!ok)
 		goto out;
 
@@ -135,6 +138,9 @@ out:
 	modem = ofono_gprs_context_get_modem(gc);
 	interface = ofono_modem_get_string(modem, "NetworkInterface");
 
+	ofono_info("IP: %s  Gateway: %s", ip, gateway);
+	ofono_info("DNS: %s, %s", dns[0], dns[1]);
+
 	CALLBACK_WITH_SUCCESS(gcd->up_cb, interface, success, ip,
 					STATIC_IP_NETMASK, gateway,
 					success ? dns : NULL, gcd->cb_data);
@@ -147,7 +153,10 @@ static void mbm_get_ip_details(struct ofono_gprs_context *gc)
 {
 	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
 	struct ofono_modem *modem;
-	const char *interface;	
+	const char *interface;
+	char buf[64];
+
+	DBG("");
 
 	if (gcd->have_e2ipcfg) {
 		g_at_chat_send(gcd->chat, "AT*E2IPCFG?", e2ipcfg_prefix,
@@ -155,6 +164,9 @@ static void mbm_get_ip_details(struct ofono_gprs_context *gc)
 		return;
 	}
 
+	snprintf(buf, sizeof(buf), "AT+CGPADDR=%u", gcd->active_context);
+	g_at_chat_send(gcd->chat, buf, none_prefix, NULL, NULL, NULL);
+
 	modem = ofono_gprs_context_get_modem(gc);
 	interface = ofono_modem_get_string(modem, "NetworkInterface");
 	CALLBACK_WITH_SUCCESS(gcd->up_cb, interface, FALSE, NULL, NULL,
@@ -169,12 +181,14 @@ static void mbm_state_changed(struct ofono_gprs_context *gc, int state)
 {
 	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
 
+	DBG("state %d", state);
+
 	if (gcd->active_context == 0)
 		return;
 
 	switch (state) {
 	case MBM_E2NAP_DISCONNECTED:
-		ofono_debug("MBM Context: disconnected");
+		DBG("disconnected");
 
 		if (gcd->mbm_state == MBM_DISABLING) {
 			CALLBACK_WITH_SUCCESS(gcd->down_cb, gcd->cb_data);
@@ -194,7 +208,7 @@ static void mbm_state_changed(struct ofono_gprs_context *gc, int state)
 		break;
 
 	case MBM_E2NAP_CONNECTED:
-		ofono_debug("MBM Context: connected");
+		DBG("connected");
 
 		if (gcd->mbm_state == MBM_ENABLING)
 			mbm_get_ip_details(gc);
@@ -202,7 +216,7 @@ static void mbm_state_changed(struct ofono_gprs_context *gc, int state)
 		break;
 
 	case MBM_E2NAP_CONNECTING:
-		ofono_debug("MBM Context: connecting");
+		DBG("connecting");
 		break;
 
 	default:
@@ -219,6 +233,8 @@ static void mbm_enap_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	GAtResultIter iter;
 	int state;
 
+	DBG("ok %d", ok);
+
 	g_at_result_iter_init(&iter, result);
 
 	if (g_at_result_iter_next(&iter, "*ENAP:") == FALSE)
@@ -253,13 +269,15 @@ static void at_enap_down_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_gprs_context *gc = cbd->user;
 	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
 	struct ofono_error error;
-	
+
+	DBG("ok %d", ok);
+
 	/* Now we have to wait for the unsolicited notification to arrive */
 	if (ok && gcd->enap != 0) {
 		gcd->mbm_state = MBM_DISABLING;
 		gcd->down_cb = cb;
 		gcd->cb_data = cbd->data;
-		
+
 		if (gcd->have_e2nap == FALSE)
 			g_at_chat_send(gcd->chat, "AT*ENAP?", enap_prefix,
 					mbm_enap_poll_cb, gc, NULL);
@@ -279,6 +297,8 @@ static void mbm_enap_up_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
 	struct ofono_error error;
 
+	DBG("ok %d", ok);
+
 	if (ok) {
 		gcd->mbm_state = MBM_ENABLING;
 		gcd->up_cb = cb;
@@ -306,7 +326,7 @@ static void mbm_cgdcont_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct cb_data *ncbd;
 	char buf[64];
 
-	dump_response("cgdcont_cb", ok, result);
+	DBG("ok %d", ok);
 
 	if (!ok) {
 		struct ofono_error error;
@@ -320,14 +340,13 @@ static void mbm_cgdcont_cb(gboolean ok, GAtResult *result, gpointer user_data)
 
 	ncbd = g_memdup(cbd, sizeof(struct cb_data));
 
-	sprintf(buf, "AT*ENAP=1,%u", gcd->active_context);
+	snprintf(buf, sizeof(buf), "AT*ENAP=1,%u", gcd->active_context);
 
 	if (g_at_chat_send(gcd->chat, buf, none_prefix,
 				mbm_enap_up_cb, ncbd, g_free) > 0)
 		return;
 
-	if (ncbd)
-		g_free(ncbd);
+	g_free(ncbd);
 
 	gcd->active_context = 0;
 
@@ -343,6 +362,8 @@ static void mbm_gprs_activate_primary(struct ofono_gprs_context *gc,
 	char buf[AUTH_BUF_LENGTH];
 	int len;
 
+	DBG("cid %u", ctx->cid);
+
 	if (!cbd)
 		goto error;
 
@@ -350,25 +371,30 @@ static void mbm_gprs_activate_primary(struct ofono_gprs_context *gc,
 
 	cbd->user = gc;
 
-	sprintf(buf, "AT*EIAAUW=%d,1,\"%s\",\"%s\"", ctx->cid,
-		ctx->username, ctx->password);
-
-	if (g_at_chat_send(gcd->chat, buf, none_prefix,
-				NULL, NULL, NULL) == 0)
-		goto error;
-						
-	len = sprintf(buf, "AT+CGDCONT=%u,\"IP\"", ctx->cid);
+	len = snprintf(buf, sizeof(buf), "AT+CGDCONT=%u,\"IP\"", ctx->cid);
 
 	if (ctx->apn)
 		snprintf(buf + len, sizeof(buf) - len - 3, ",\"%s\"",
 				ctx->apn);
 
 	if (g_at_chat_send(gcd->chat, buf, none_prefix,
-				mbm_cgdcont_cb, cbd, g_free) > 0)
-		return;
+				mbm_cgdcont_cb, cbd, g_free) == 0)
+		goto error;
+
+	/*
+	 * Set username and password, this should be done after CGDCONT
+	 * or an error can occur.  We don't bother with error checking
+	 * here
+	 * */
+	snprintf(buf, sizeof(buf), "AT*EIAAUW=%d,1,\"%s\",\"%s\"",
+			ctx->cid, ctx->username, ctx->password);
+
+	g_at_chat_send(gcd->chat, buf, none_prefix, NULL, NULL, NULL);
+
+	return;
+
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, NULL, 0, NULL, NULL, NULL, NULL, data);
 }
@@ -380,6 +406,8 @@ static void mbm_gprs_deactivate_primary(struct ofono_gprs_context *gc,
 	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
+	DBG("cid %u", cid);
+
 	if (!cbd)
 		goto error;
 
@@ -390,8 +418,7 @@ static void mbm_gprs_deactivate_primary(struct ofono_gprs_context *gc,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -417,6 +444,8 @@ static void mbm_e2nap_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_gprs_context *gc = user_data;
 	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
 
+	DBG("ok %d", ok);
+
 	gcd->have_e2nap = ok;
 
 	if (ok)
@@ -439,13 +468,22 @@ static int mbm_gprs_context_probe(struct ofono_gprs_context *gc,
 	GAtChat *chat = data;
 	struct gprs_context_data *gcd;
 
-	gcd = g_new0(struct gprs_context_data, 1);
-	gcd->chat = chat;
+	DBG("");
+
+	gcd = g_try_new0(struct gprs_context_data, 1);
+	if (!gcd)
+		return -ENOMEM;
+
+	gcd->chat = g_at_chat_clone(chat);
 
 	ofono_gprs_context_set_data(gc, gcd);
 
-	g_at_chat_send(chat, "AT*E2NAP=1", none_prefix, mbm_e2nap_cb, gc, NULL);
-	g_at_chat_send(chat, "AT*E2IPCFG=?", e2ipcfg_prefix,
+	g_at_chat_send(gcd->chat, "AT*ENAPDBG=1", none_prefix,
+				NULL, NULL, NULL);
+
+	g_at_chat_send(gcd->chat, "AT*E2NAP=1", none_prefix,
+			mbm_e2nap_cb, gc, NULL);
+	g_at_chat_send(gcd->chat, "AT*E2IPCFG=?", e2ipcfg_prefix,
 			mbm_e2ipcfg_query_cb, gc, NULL);
 
 	return 0;
@@ -455,17 +493,21 @@ static void mbm_gprs_context_remove(struct ofono_gprs_context *gc)
 {
 	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
 
+	DBG("");
+
 	if (gcd->enap_source) {
 		g_source_remove(gcd->enap_source);
 		gcd->enap_source = 0;
 	}
 
 	ofono_gprs_context_set_data(gc, NULL);
+
+	g_at_chat_unref(gcd->chat);
 	g_free(gcd);
 }
 
 static struct ofono_gprs_context_driver driver = {
-	.name			= "mbm",
+	.name			= "mbmmodem",
 	.probe			= mbm_gprs_context_probe,
 	.remove			= mbm_gprs_context_remove,
 	.activate_primary	= mbm_gprs_activate_primary,
